// ------------------------------------------------------------------------------------
// <copyright file="LibrosBL.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.Negocio.Clases.BL
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Biblioteca.Datos.Clases.DAL.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.IC.DTO.Repositorio;
	using Biblioteca.IC.Recursos;
	using TransaccionalNet.EF.Clases;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using Utilitarios.TransversalesNet.Clases.Comunes.Enumeraciones;

	/// <summary>
	/// Clase con las acciones de negocio de la entidad Libros
	/// </summary>
	public class LibrosBL : AccesoComunBL, ILibrosNegocioAccion
	{

		#region ATRIBUTOS
		/// <summary>
		/// Objeto para repositorio Libros
		/// <summary>
		/// Objeto para acciones Libros
		/// </summary>
		private Lazy<ILibrosRepositorioAccion> librosRepositorioAccion;

		/// <summary>
		/// Objeto para entidad respuesta
		/// </summary>
		Respuesta<ILibrosDTO> respuesta;

		#endregion ATRIBUTOS
		#region CONSTRUCTORES
		/// <summary>
		/// Inicializa una nueva instancia de la clase LibrosBL
		/// </summary>
		/// <param name="argLibrosRepositorioAccion">Acciones entidad Libros</param>
		public LibrosBL(Lazy<ILibrosRepositorioAccion> argLibrosRepositorioAccion = null)
		{
			this.respuesta = new Respuesta<ILibrosDTO>();
			this.librosRepositorioAccion = argLibrosRepositorioAccion ?? new Lazy<ILibrosRepositorioAccion>(() => new LibrosDAL());
		}

		#endregion CONSTRUCTORES
		#region METODOS PUBLICOS
		/// <summary>
		/// Metodo consultar lista libros
		/// </summary>
		/// <returns>Respuesta tipo Libros </returns>
		public async Task<Respuesta<ILibrosDTO>> ConsultarListaLibrosAsync()
		{

			return await this.EjecutarTransaccionBDAsync<Respuesta<ILibrosDTO>, LibrosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.librosRepositorioAccion.Value.ConsultarListaLibrosAsync();
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});

		}
		/// <summary>
		/// Metodo consultar por llave libros
		/// </summary>
		/// <param name="libros">Entidad a consultar</param>
		/// <returns>Respuesta tipo Libros </returns>
		public async Task<Respuesta<ILibrosDTO>> ConsultarLibrosLlaveAsync(ILibrosDTO libros)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ILibrosDTO>, LibrosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.librosRepositorioAccion.Value.ConsultarLibrosLlaveAsync(libros);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo editar libros
		/// </summary>
		/// <param name="libros">Entidad a editar</param>
		/// <returns>Respuesta tipo Libros </returns>
		public async Task<Respuesta<ILibrosDTO>> EditarLibrosAsync(ILibrosDTO libros)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ILibrosDTO>, LibrosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.librosRepositorioAccion.Value.EditarLibrosAsync(libros);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}


		/// <summary>
		/// Metodo eliminar libros
		/// </summary>
		/// <param name="libros">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Libros </returns>
		public async Task<Respuesta<ILibrosDTO>> EliminarLibrosAsync(ILibrosDTO libros)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ILibrosDTO>, LibrosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.librosRepositorioAccion.Value.EliminarLibrosAsync(libros);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeEntidadEliminadaConExito);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo guardar libros
		/// </summary>
		/// <param name="libros">Entidad a guardar</param>
		/// <returns>Respuesta tipo Libros </returns>
		public async Task<Respuesta<ILibrosDTO>> GuardarLibrosAsync(ILibrosDTO libros)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ILibrosDTO>, LibrosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.librosRepositorioAccion.Value.GuardarLibrosAsync(libros);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}

		/// <summary>
		/// Metodo Guardar lista libros
		/// </summary>
		/// <param name="listaLibros">Lista de entidades a guardar</param>
		/// <returns>Respuesta tipo Libros </returns>
		public async Task<Respuesta<ILibrosDTO>> GuardarListaLibrosAsync(List<ILibrosDTO> listaLibros)
		{
			return await this.EjecutarTransaccionBDAsync<Respuesta<ILibrosDTO>, LibrosBL>(
			System.Transactions.IsolationLevel.ReadUncommitted,
			async () =>
			{
				 respuesta= await this.librosRepositorioAccion.Value.GuardarListaLibrosAsync(listaLibros);
				 respuesta.Resultado = true;
				 respuesta.TipoNotificacion = TipoNotificacion.Exitoso;
				 respuesta.Mensajes.Add(rcsMensajesComunes.MensajeCreacionEdicionExitosa);
				 return respuesta;

			});
		}
		#endregion METODOS PUBLICOS
		#region METODOS PRIVADOS
		#endregion METODOS PRIVADOS
	}
}

