// ------------------------------------------------------------------------------------
// <copyright file="LibrosController.cs" company="Universidad Distrital">
// Copyright (c) Universidad Distrital. All rights reserved.
// </copyright>
// <author>Camilo Prieto</author>
// ------------------------------------------------------------------------------------
namespace Biblioteca.API.Controllers
{
	using System;
	using System.Collections.Generic;
	using System.Threading.Tasks;
	using Utilitarios.APINet.Clases.Ejecucion;
	using UtilitariosNet.Clases.Comunes.Entidades;
	using UtilitariosNet.Clases.Mapeador.Extensiones;
	using Biblioteca.API.Models.Repositorio;
	using Biblioteca.IC.Acciones.Repositorio;
	using Biblioteca.Negocio.Clases.BL;
	using Microsoft.AspNetCore.Mvc;
	
	/// <summary>
	/// Clase con las capaciades Rest de la entidad Libros
	/// </summary>
	[Route("api/Libros")]
	public class LibrosController : AccesoComunAPI
	{


		/// <summary>
		/// Objeto para negocio Libros
		/// </summary>
		private Lazy<ILibrosNegocioAccion> negocioLibros;

		
		/// <summary>
		/// Inicializa una nueva instancia de la clase LibrosController
		/// </summary>
		public LibrosController()
		{
			this.negocioLibros = new Lazy<ILibrosNegocioAccion>(
										() => new LibrosBL());
		}
		
		/// <summary>
		/// Metodo consultar lista libros
		/// </summary>
		/// <returns>Respuesta tipo Libros </returns>
		[HttpGet]
		[Route("ConsultarListaLibros")]
		public async Task<Respuesta<Libros>> ConsultarListaLibros()
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Libros>>, LibrosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Libros>>(await negocioLibros.Value.ConsultarListaLibrosAsync());
			});
		}
		
		/// <summary>
		/// Metodo consultar por llave libros
		/// </summary>
		/// <param name="libros">Entidad a consultar</param>
		/// <returns>Respuesta tipo Libros </returns>
		[HttpPost]
		[Route("ConsultarLibrosLlave")]
		public async Task<Respuesta<Libros>> ConsultarLibrosLlave(Libros libros)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Libros>>, LibrosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Libros>>(await negocioLibros.Value.ConsultarLibrosLlaveAsync(libros));
			});
		}
		
		/// <summary>
		/// Metodo guardar libros
		/// </summary>
		/// <param name="libros">Entidad a guardar</param>
		/// <returns>Respuesta tipo Libros </returns>
		[HttpPost]
		[Route("GuardarLibros")]
		public async Task<Respuesta<Libros>> GuardarLibros(Libros libros)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Libros>>, LibrosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Libros>>(await negocioLibros.Value.GuardarLibrosAsync(libros));
			});
		}
		
		/// <summary>
		/// Metodo editar libros
		/// </summary>
		/// <param name="libros">Entidad a editar</param>
		/// <returns>Respuesta tipo Libros </returns>
		[HttpPut]
		[Route("EditarLibros")]
		public async Task<Respuesta<Libros>> EditarLibros(Libros libros)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Libros>>, LibrosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Libros>>(await negocioLibros.Value.EditarLibrosAsync(libros));
			});
		}
		
		/// <summary>
		/// Metodo eliminar libros
		/// </summary>
		/// <param name="libros">Entidad a eliminar</param>
		/// <returns>Respuesta tipo Libros </returns>
		[HttpDelete]
		[Route("EliminarLibros")]
		public async Task<Respuesta<Libros>> EliminarLibros(Libros libros)
		{
			return await this.EjecutarTransaccionAPI<Task<Respuesta<Libros>>, LibrosController>(async () =>
			{
				return Mapeador.MapearObjetoPorJson<Respuesta<Libros>>(await negocioLibros.Value.EliminarLibrosAsync(libros));
			});
		}
	}
}

